/*
 * checkpris -	Check up/down status of PRIs on Ascend MAX and 3COM
 *		HiperDSP cards
 *
 * VERSION 1.0.1    - Nov 22, 2000
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>

/* START OF USER DEFINABLE SETTINGS */
/* Below are all the definitions that you will need to modify		*/
/* that usually dont change. Rest of them are in the config file	*/

/* Path to data file							*/
#define DATAFILE "/etc/checkpris.cfg"

/* Directory where status data files will get stored for checking	*/
/* Filename format: DATADIR/.pri-<host>-<pri #>.dat			*/
#define DATADIR "/tmp"

/* File to log info/errors to						*/
#define LOGFILE "/tmp/.checkpris.log"

/* Path to the sendmail program to send out notifications		*/
#define PATHTO  "/usr/sbin/sendmail -t"

/* OID for operational status of Ascend MAX T1 Span */
#define LINESTATE_OID_M	"interfaces.ifTable.ifEntry.ifOperStatus.%d"

/* OID for operational status of 3COM T1 Span, interfaces are		*/
/* <slot #>025 as in slot 1 is interface 1025, 2 is 2025, 10 is 10025	*/
#define LINESTATE_OID_3	".1.3.6.1.2.1.10.18.6.1.10.%d"

/* OIDs for yellow alarm and red alarm state of 3com Netserver type PRIs */
/* State of 1 means NO alarm. Interfaces are 100<span #> as in span 1 is */
/* 1001									*/
#define LINESTATE_OID_NETSRV_YELLOW ".1.3.6.1.3.2.1.1.9.%d"
#define LINESTATE_OID_NETSRV_RED ".1.3.6.1.3.2.1.1.10.%d"

/* Program to get snmp requests						*/
#define SNMP_PROG	"/usr/bin/snmpget"

/* Read SNMP community of chassises we are checking			*/
#define READ_COMM	"public"

/* Who do we set the from address to in an email or page?		*/
#define FROM    "nobody@yourdomain.com"

/* Who to page. Normally an email address that pages an admin via modem	*/
/* this goes along with P_SUBJECT which I set to blank for paging	*/
/* Set this to nothing to disable					*/
#define PAGETO  "admin-pager@yourdomain.com"

/* First person to email notification with a subject of M_SUBJECT	*/
/* Set this to nothing to disable					*/
#define MAILTO1 "admin@yourdomain.com"

/* Second person to email notification with a subject of M_SUBJECT	*/
/* Set this to nothing to disable					*/
#define MAILTO2 "admin2@yourdomain.com"

/* Subject of page, if you want one					*/
#define P_SUBJECT       ""
/* Subject of email, if you want one					*/
#define M_SUBJECT       "PRI STATUS"

/* Print debugging info?  1 for yes, 0 for no				*/
#define DEBUG 0

/* END OF USER DEFINABLE SETTINGS */


void wlog(char *str);
void remove_first_comma(char *inpstr);
void notify_admin(char *host, char *desc, char *type, int port, char *status);
void midcpy(char *strf, char *strt, int fr, int to);
void remove_first(char *inpstr);
void strtolower(char *str);

char mess[500];

int main(void)
{
int i=0,j=0,z=0;
int wanport[14];
char hostname[50];
char hostdesc[50];
char hosttype[50];
char temp[3];
char oid[256];
char junk1[100];
char junk2[100];
char junk3[100];
char junk4[100];
char line[257];
char wanline[257];
char command[256];
char filename[256];
FILE *fp;
FILE *fp2;
FILE *pp;

hostname[0]=0;
wanport[0]=0;
wanport[1]=0;
wanport[2]=0;
wanport[3]=0;
wanport[4]=0;
wanport[5]=0;
wanport[6]=0;
wanport[7]=0;
wanport[8]=0;
wanport[9]=0;
wanport[10]=0;
wanport[11]=0;
wanport[12]=0;
wanport[13]=0;
line[0]=0;

if (!(fp=fopen(DATAFILE,"r"))) {
	sprintf(mess,"Can't open checkpri data file %s to get list!\n",DATAFILE);
	printf("%s",mess);
	wlog(mess);
	sprintf(mess,"ERRNO: %s (%d)\n",strerror(errno),errno);
	printf("%s",mess);
	wlog(mess);
	exit(0);
	}

        while (fgets(line,256,fp) != NULL) {
                line[strlen(line)-1]=0;
		if (line[0]=='#' || line[0]==' ' || !strlen(line)) continue;
		sscanf(line,"%s ",hostname);
		remove_first(line);
		sscanf(line,"%s ",hostdesc);
		remove_first(line);
		sscanf(line,"%s ",hosttype);
		remove_first(line);
		sscanf(line,"%s ",wanline);

if (DEBUG==1)	{ printf("Hostname: %s\nDesc: %s\nType: %s\nWanline: %s\n",
		hostname,hostdesc,hosttype,wanline); }

                if (strstr(wanline,",")) {
                for (i=0;i<strlen(wanline);++i) {
                        if (wanline[i]==',') j++;
                   }
                i=0;
                for (i=0;i<j+1;++i) {
			/* find comma */
			for (z=0;z<strlen(wanline);++z) {
                        if (wanline[z]==',') break;
                        }
                        midcpy(wanline,temp,0,z);
			z=0;
                        wanport[i]=atoi(temp);

if (DEBUG==1)		printf("Set wanport %d to %d\n",i,wanport[i]);

                        if (strstr(wanline,","))
                        remove_first_comma(wanline);
                   }
                }
                else {
                        wanport[i]=atoi(wanline);
                }

if (DEBUG==1) {
printf("Wanports: %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
wanport[0],wanport[1],wanport[2],wanport[3],
wanport[4],wanport[5],wanport[6],wanport[7]
,wanport[8],wanport[9],wanport[10],wanport[11],wanport[12]
,wanport[13]);
}

i=0;
for (i=0;i<14;++i) {
junk3[0]=0;
if (wanport[i]==0) continue;

        /* get linestate via snmp */
	if (!strcmp(hosttype,"MAX"))
	sprintf(oid,LINESTATE_OID_M,wanport[i]+1);
	else if (!strcmp(hosttype,"3COM_HIPER"))
	sprintf(oid,LINESTATE_OID_3,(wanport[i]*1000)+25);
	else if (!strcmp(hosttype,"3COM_NETSV"))
	sprintf(oid,LINESTATE_OID_NETSRV_YELLOW,1000+wanport[i]);

        sprintf(command,"%s %s %s %s",SNMP_PROG,hostname,READ_COMM,oid);
if (DEBUG==1) printf("Running %s\n",command);

        if (!(pp=popen(command,"r"))) {
                strcpy(mess,"Can't open LINESTATE pipe!\n");
		printf("%s",mess);
		wlog(mess);
		continue;
                }
	line[0]=0;
        fgets(line,256,pp);
        pclose(pp);
        line[strlen(line)-1]=0;
if (DEBUG==1) printf("Got SNMP line %s\n",line);
        sscanf(line,"%s %s %s",junk1,junk2,junk3);

if (!strcmp(hosttype,"3COM_NETSV")) {
/* get the red alarm status too */
	sprintf(oid,LINESTATE_OID_NETSRV_RED,1000+wanport[i]);
        sprintf(command,"%s %s %s %s",SNMP_PROG,hostname,READ_COMM,oid);
if (DEBUG==1) printf("Running %s\n",command);

        if (!(pp=popen(command,"r"))) {
                strcpy(mess,"Can't open LINESTATE pipe!\n");
		printf("%s",mess);
		wlog(mess);
		continue;
                }
        fgets(line,256,pp);
        pclose(pp);
        line[strlen(line)-1]=0;
        sscanf(line,"%s %s %s",junk1,junk2,junk4);

	if (strlen(junk3)) {
	strcat(junk3," ");
	strcat(junk3,junk4);
	}
} /* if netserver */

	/* open status file for this host and line */
	/* to check for a match			   */
	sprintf(filename,"%s/.pri-%s-%d.dat",DATADIR,hostname,wanport[i]);

	if (!(fp2=fopen(filename,"r"))) {
if (DEBUG==1)	printf("Can't read %s, writing new status %s\n",filename,junk3);
	fp2=fopen(filename,"w");
	fputs(junk3,fp2);
	fputs("\n",fp2);
	fclose(fp2);
	}
	else {
	fgets(line,256,fp2);
	line[strlen(line)-1]=0;
	if (!strlen(junk3)) strcpy(junk3,"cantget");

		if (!strcmp(line,junk3)) {
		/* status is same, no need to page or write new data */
		fclose(fp2);
if (DEBUG==1)		printf("Status SAME in %s\n",filename);
		continue;
		}
		else {
		/* status is not same, we will update file and */
		/* page new status				*/
		fclose(fp2);
if (DEBUG==1)		printf("Status CHANGED in %s to %s\n",filename,junk3);
		fp2=fopen(filename,"w");
		fputs(junk3,fp2);
		fputs("\n",fp2);
		fclose(fp2);
		notify_admin(hostname,hostdesc,hosttype,wanport[i],junk3);
		}
	} /* end of main else */
	continue;

} /* end of for */
i=0;
j=0;
hostname[0]=0;
hostdesc[0]=0;
hosttype[0]=0;
wanport[0]=0;
wanport[1]=0;
wanport[2]=0;
wanport[3]=0;
wanport[4]=0;
wanport[5]=0;
wanport[6]=0;
wanport[7]=0;
wanport[8]=0;
wanport[9]=0;
wanport[10]=0;
wanport[11]=0;
wanport[12]=0;
wanport[13]=0;
line[0]=0;
} /* end of file while */
fclose(fp);

return 1;
} /* end of main */

/*** convert string to lower case ***/
void strtolower(char *str)
{
while(*str)
  {
    *str=tolower((int)*str);
    str++;
  }
}

/*** removes first word at front of string and moves rest down ***/
void remove_first(char *inpstr)
{
int newpos,oldpos;
        
newpos=0;  oldpos=0;
/* find first word */ 
while(inpstr[oldpos]==' ') {
        if (!inpstr[oldpos]) { inpstr[0]=0;  return; }
        oldpos++;
        }
/* find end of first word */
while(inpstr[oldpos]!=' ') {
        if (!inpstr[oldpos]) { inpstr[0]=0;  return; }
        oldpos++;
        }         
/* find second word */
while(inpstr[oldpos]==' ') {
        if (!inpstr[oldpos]) { inpstr[0]=0;  return; }
        oldpos++;
        }
while(inpstr[oldpos]!=0)
        inpstr[newpos++]=inpstr[oldpos++];
inpstr[newpos]='\0';
}

/*** removes first word at front of string and moves rest down ***/
void remove_first_comma(char *inpstr)
{
int newpos,oldpos;
newpos=0;  oldpos=0;
/* find first word */
while(inpstr[oldpos]==',') {
        if (!inpstr[oldpos]) { inpstr[0]=0;  return; }
        oldpos++;
        }
/* find end of first word */
while(inpstr[oldpos]!=',') {
        if (!inpstr[oldpos]) { inpstr[0]=0;  return; }
        oldpos++;
        }
/* find second word */
while(inpstr[oldpos]==',') {
        if (!inpstr[oldpos]) { inpstr[0]=0;  return; }
        oldpos++;
        }
while(inpstr[oldpos]!=0)
        inpstr[newpos++]=inpstr[oldpos++];
inpstr[newpos]='\0';
}

void midcpy(char *strf, char *strt, int fr, int to)
{
int f;
for (f=fr;f<=to;++f)
  {
   if (!strf[f])
     {
      strt[f-fr]='\0';
      return;
     }
   strt[f-fr]=strf[f];
  }
strt[f-fr]='\0';
}

void wlog(char *str)
{
FILE *fp;
        
fp=fopen(LOGFILE,"a");
fputs(str,fp);
fclose(fp);
}

void notify_admin(char *host, char *desc, char *type, int port, char *status)
{
FILE *pp;
char tem[256];

sprintf(tem,"%s",PATHTO);

if (strlen(PAGETO)) {
  if (!(pp=popen(tem,"w")))
    {
     exit(0);
    }
   fprintf(pp,"From: \"%s\"\n",FROM);
   fprintf(pp,"To: %s\n",PAGETO);
   fprintf(pp,"Subject: %s\n\n",P_SUBJECT);
   fprintf(pp,"PRI %d on %s %s (%s) has new status of %s",port,type,desc,host,status);
   fputs("\n\n",pp);
   fputs(".\n",pp);
   pclose(pp);
}

if (strlen(MAILTO1)) {
  if (!(pp=popen(tem,"w")))
    {
     exit(0);
    }
   fprintf(pp,"From: \"%s\"\n",FROM);
   fprintf(pp,"To: %s\n",MAILTO1);
   fprintf(pp,"Subject: %s\n\n",M_SUBJECT);
   fprintf(pp,"PRI %d on %s %s (%s) has new status of %s",port,type,desc,host,status);
   fputs("\n\n",pp);
   fputs(".\n",pp);
   pclose(pp);
}

if (strlen(MAILTO2)) {
  if (!(pp=popen(tem,"w")))
    {
     exit(0);
    }
   fprintf(pp,"From: \"%s\"\n",FROM);
   fprintf(pp,"To: %s\n",MAILTO2);
   fprintf(pp,"Subject: %s\n\n",M_SUBJECT);
   fprintf(pp,"PRI %d on %s %s (%s) has new status of %s",port,type,desc,host,status);
   fputs("\n\n",pp);
   fputs(".\n",pp);
   pclose(pp);
}

}

